@@ -6,20 +6,24 @@ use crate::filename::MarkupLanguage;
 use crate::viewer::error::ViewerError;
 use crate::viewer::sse_server::manage_connections;
 use crate::viewer::watcher::FileWatcher;
+use crate::viewer::watcher::Mode;
 use crate::viewer::web_browser::launch_web_browser;
 use std::net::TcpListener;
 use std::path::PathBuf;
 use std::sync::{Arc, Mutex};
 use std::thread;
-use std::thread::sleep;
 use std::thread::JoinHandle;
-use std::time::{Duration, Instant};
+use std::time::Instant;
 
-/// Normally the viewer keeps running until the browser process finishes. This
-/// happens usually when the user closes the browser window.  In case the
-/// process returns immediately (when it forks), we keep the viewer up and
-/// running at least some seconds.
-const VIEWER_MIN_UPTIME: u64 = 4;
+/// Time span in seconds between extra `update()` to check if the
+/// browser is still connected.
+const WATCHDOG_INTERVAL: u64 = 60;
+
+/// Minimum uptime in milliseconds we expect a real browser instance to run.
+/// When starting a second browser instance, only a signal is sent to the
+/// first instance and the process returns immediately. We detect this
+/// case if it runs less milliseconds than this constant.
+const BROWSER_INSTANCE_MIN_UPTIME: u128 = 3000;
 
 /// This is where our loop back device is.
 /// The following is also possible, but binds us to IPv4:
@@ -101,32 +105,35 @@ impl Viewer {
         // Launch the file watcher thread.
         // Send a signal whenever the file is modified. Without error, this thread runs as long as
         // the parent thread (where we are) is running.
-        let _handle: JoinHandle<_> = thread::spawn(move || loop {
-            match FileWatcher::new(doc.clone(), event_tx_list.clone()) {
-                Ok(mut w) => w.run(),
-                Err(e) => {
-                    log::warn!("Can not (re-)start file watcher, giving up: {}", e);
-                    break;
-                }
-            }
-        });
+        let mode = Arc::new(Mutex::new(Mode::OneTick(WATCHDOG_INTERVAL)));
+        let mode_clone = mode.clone();
+        let watcher_handle: JoinHandle<_> =
+            thread::spawn(
+                move || match FileWatcher::new(doc, event_tx_list, mode_clone) {
+                    Ok(mut w) => w.run(),
+                    Err(e) => {
+                        log::warn!("Can not start file watcher, giving up: {}", e);
+                    }
+                },
+            );
 
         // Launch web browser.
         let url = format!("http://{}:{}", LOCALHOST, localport);
         log::info!("Viewer::run(): launching browser with URL: {}", url);
 
         // Start timer.
-        let now = Instant::now();
-        // This blocks until the browser is closed.
+        let browser_start = Instant::now();
+
         launch_web_browser(&url)?;
 
-        // If ever the browsers thread does not block (it should), then we wait a little to
-        // serve the rendered page and the referenced images.
-        if now.elapsed().as_secs() <= VIEWER_MIN_UPTIME {
-            sleep(Duration::new(4, 0));
-        };
+        if browser_start.elapsed().as_millis() < BROWSER_INSTANCE_MIN_UPTIME {
+            // We are there because the browser process did not block.
+            // We enable the watchdog, ...
+            *mode.lock().unwrap() = Mode::Ticks(WATCHDOG_INTERVAL);
+            // ... and wait until the watchdog recognizes, that the web browser disconnected.
+            watcher_handle.join().unwrap();
+        }
 
-        // This will also terminate the `FileWatcher` thread and web server thread.
         Ok(())
     }
 }
